/**
* @license Apache-2.0
*
* Copyright (c) 2018 The Stdlib Authors.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

'use strict';

// MODULES //

var tape = require( 'tape' );
var isnan = require( '@stdlib/math/base/assert/is-nan' );
var randuFactory = require( '@stdlib/random/base/randu' ).factory;
var pdf = require( './../../../../../base/dists/kumaraswamy/pdf' );
var abs = require( '@stdlib/math/base/special/abs' );
var ln = require( '@stdlib/math/base/special/ln' );
var PINF = require( '@stdlib/constants/float64/pinf' );
var NINF = require( '@stdlib/constants/float64/ninf' );
var EPS = require( '@stdlib/constants/float64/eps' );
var logpdf = require( './../lib' );


// TESTS //

tape( 'main export is a function', function test( t ) {
	t.ok( true, __filename );
	t.equal( typeof logpdf, 'function', 'main export is a function' );
	t.end();
});

tape( 'if provided `NaN` for any parameter, the function returns `NaN`', function test( t ) {
	var y = logpdf( NaN, 0.0, 1.0 );
	t.equal( isnan( y ), true, 'returns NaN' );
	y = logpdf( 0.0, NaN, 1.0 );
	t.equal( isnan( y ), true, 'returns NaN' );
	y = logpdf( 0.0, 1.0, NaN );
	t.equal( isnan( y ), true, 'returns NaN' );
	t.end();
});

tape( 'if provided a number greater than or equal to one for `x` and a finite `a` and `b`, the function returns `-infinity`', function test( t ) {
	var y = logpdf( PINF, 0.5, 1.0 );
	t.equal( y, NINF, 'returns -infinity' );

	y = logpdf( 100.0, 0.5, 1.0 );
	t.equal( y, NINF, 'returns -infinity' );

	y = logpdf( 10.0, 0.5, 1.0 );
	t.equal( y, NINF, 'returns -infinity' );

	y = logpdf( 1.0, 0.5, 1.0 );
	t.equal( y, NINF, 'returns -infinity' );

	t.end();
});

tape( 'if provided a number less than or equal to zero for `x` and a finite `a` and `b`, the function returns `-infinity`', function test( t ) {
	var y = logpdf( NINF, 0.5, 1.0 );
	t.equal( y, NINF, 'returns -infinity' );

	y = logpdf( -100.0, 0.5, 1.0 );
	t.equal( y, NINF, 'returns -infinity' );

	y = logpdf( -1.0, 0.5, 1.0 );
	t.equal( y, NINF, 'returns -infinity' );

	y = logpdf( 0.0, 0.5, 1.0 );
	t.equal( y, NINF, 'returns -infinity' );

	t.end();
});

tape( 'if provided a nonpositive `a`, the function returns `NaN`', function test( t ) {
	var y;

	y = logpdf( 2.0, 0.0, 2.0 );
	t.equal( isnan( y ), true, 'returns NaN' );

	y = logpdf( 2.0, -1.0, 2.0 );
	t.equal( isnan( y ), true, 'returns NaN' );

	y = logpdf( 0.0, -1.0, 2.0 );
	t.equal( isnan( y ), true, 'returns NaN' );

	y = logpdf( 2.0, NINF, 1.0 );
	t.equal( isnan( y ), true, 'returns NaN' );

	y = logpdf( 2.0, NINF, PINF );
	t.equal( isnan( y ), true, 'returns NaN' );

	y = logpdf( 2.0, NINF, NINF );
	t.equal( isnan( y ), true, 'returns NaN' );

	y = logpdf( 2.0, NINF, NaN );
	t.equal( isnan( y ), true, 'returns NaN' );

	t.end();
});

tape( 'if provided a nonpositive `b`, the function returns `NaN`', function test( t ) {
	var y;

	y = logpdf( 2.0, 2.0, 0.0 );
	t.equal( isnan( y ), true, 'returns NaN' );

	y = logpdf( 2.0, 2.0, -1.0 );
	t.equal( isnan( y ), true, 'returns NaN' );

	y = logpdf( 0.0, 2.0, -1/0 );
	t.equal( isnan( y ), true, 'returns NaN' );

	y = logpdf( 2.0, 1.0, NINF );
	t.equal( isnan( y ), true, 'returns NaN' );

	y = logpdf( 2.0, PINF, NINF );
	t.equal( isnan( y ), true, 'returns NaN' );

	y = logpdf( 2.0, NINF, NINF );
	t.equal( isnan( y ), true, 'returns NaN' );

	y = logpdf( 2.0, NaN, NINF );
	t.equal( isnan( y ), true, 'returns NaN' );

	t.end();
});

tape( 'the function evaluates the logpdf of a Kumaraswamy\'s double bounded distribution', function test( t ) {
	var expected;
	var delta;
	var randu;
	var tol;
	var a;
	var b;
	var x;
	var y;
	var i;

	randu = randuFactory({
		'seed': 240
	});
	for ( i = 0; i < 100; i++ ) {
		x = randu();
		a = ( randu()*5.0 ) + EPS;
		b = ( randu()*5.0 ) + EPS;
		y = logpdf( x, a, b );
		expected = ln( pdf( x, a, b ) );
		if ( y === expected ) {
			t.equal( y, expected, 'x: '+x+', a:'+a+', b: '+b+', y: '+y+', expected: '+expected );
		} else {
			delta = abs( y - expected );
			tol = 1000.0 * EPS * abs( expected );
			t.ok( delta <= tol, 'within tolerance. x: '+x+'. a: '+a+'. b: '+b+'. y: '+y+'. E: '+expected+'. Δ: '+delta+'. tol: '+tol+'.' );
		}
	}
	t.end();
});
